import { spriteOffsets, notification } from '@/application/state';

/**
 * Logger utility with consistent error format
 */
export const logger = {
  warn: (message: string, details?: any) => {
    console.warn(`Spritesheet: ${message}`, details || '');
  },
  error: (message: string, error?: any) => {
    console.error(`Spritesheet: ${message}`, error || '');
  },
};

/**
 * Safely execute a function with error handling
 */
export function safeExecute<T>(fn: () => T, errorMessage: string): T | undefined {
  try {
    return fn();
  } catch (error) {
    logger.error(errorMessage, error);
    return undefined;
  }
}

/**
 * Show a notification
 */
export function showNotification(message: string, type: 'success' | 'error' = 'success') {
  notification.message = message;
  notification.type = type;
  notification.isVisible = true;

  setTimeout(() => {
    notification.isVisible = false;
  }, 3000);
}

/**
 * Get the offset for a specific frame
 */
export function getSpriteOffset(frameIndex: number) {
  if (!spriteOffsets.has(frameIndex)) {
    spriteOffsets.set(frameIndex, { x: 0, y: 0 });
  }
  return spriteOffsets.get(frameIndex)!;
}

/**
 * Check if image is fully loaded and ready to use
 */
export function isImageReady(img: HTMLImageElement): boolean {
  return img.complete && img.naturalWidth !== 0;
}

/**
 * Get pixel-perfect coordinates aligned to pixel boundaries
 */
export function getPixelPerfectCoordinate(value: number): number {
  return Math.floor(value) + 0.5;
}
